@@ -28,90 +28,41 @@ import mimetypes
 import xml.sax
 
 #
-# You have a couple of choices for getting your individual keys in.
-#   1. You could pass them to the BitBucket constructor
-#   2. You could add them to this dictionary
-#   3. You could place them in a file called bitbucket.cfg and put that
-#      file in either your home directory or in the directory you are
-#      running the code in.  The file needs to look like this:
-#-------------------------------------------------------
-#      [DEFAULTS]
-#      AccessKeyID: YourAccessKeyHere
-#      SecretAccessKey: YourSecretAccessKeyHere
-#      Debug: False
-#-------------------------------------------------------
-AWS_DEFAULTS = {'AccessKeyID': '',
-		'SecretAccessKey': '',
-		'BucketNamePrefix': '',
-		'PageSize': 100,
-		'Debug': False}
-
-MD5Command = '/sbin/md5 -q "%s"'
+# Canned Access Policies
 #
-# Just wanted to subclass to include an implementation of HEAD
+_canned_access_policies = ('private', 'public-read',
+			   'public-read-write', 'authenticated-read')
 #
-class BBAWSAuthConnection(s3amazon.S3.AWSAuthConnection):
+# Dictionary mapping mime types to content filters
+#
+_content_filter_map = {}
 
-    BufferSize=8192
+def add_content_filter(content_type, filter):
+    _content_filter_map[content_type] = filter
+    filter.content_type = content_type
 
-    def head(self, bucket, key, headers={}):
-        return BBHeadResponse(
-	    self.make_request('HEAD', '%s/%s' % (bucket, urllib.quote_plus(key)), headers))
+def get_content_filter(content_type):
+    if _content_filter_map.has_key(content_type):
+	return _content_filter_map[content_type]
+    else:
+	return None
 
-    #
-    # This method mainly stolen from Emanuele Ruffaldi's S3 Tool
-    #
-    def put_file(self, bits, headers={}):
-	if not isinstance(bits, Bits):
-	    raise BitBucketTypeError('Must pass Bits to put_file')
-	final_headers = s3amazon.S3.merge_meta(headers, bits.metadata);
-	path = '%s/%s' % (bits.bucket.name, urllib.quote_plus(bits.key))
-	self.add_aws_auth_header(final_headers, 'PUT', path)
-	self.connection.putrequest('PUT','/'+path)
-	final_headers["Content-Length"] = bits.size
-	for key in final_headers:
-	    self.connection.putheader(key,final_headers[key])
-	self.connection.endheaders()
-	file = open(bits.filename, 'rb')
-	l = file.read(self.BufferSize)
-	while len(l) > 0:
-	    self.connection.send(l)
-	    l = file.read(self.BufferSize)
-	file.close()
-	return s3amazon.S3.Response(self.connection.getresponse())
+def remove_content_filter(content_type):
+    del _content_filter_map[content_type]
 
-    def get_file(self, bits, filename, headers={}):
-	if not isinstance(bits, Bits):
-	    raise BitBucketTypeError('Must pass Bits to get_file')
-	final_headers = s3amazon.S3.merge_meta(headers, bits.metadata);
-	path = '%s/%s' % (bits.bucket.name, urllib.quote_plus(bits.key))
-	self.add_aws_auth_header(final_headers, 'GET', path)
-	self.connection.putrequest('GET','/'+path)
-	for key in final_headers:
-	    self.connection.putheader(key,final_headers[key])
-	self.connection.endheaders()
-	resp = self.connection.getresponse()
-	response_headers = resp.msg
-	for key in response_headers.keys():
-	    if key.lower().startswith(s3amazon.S3.METADATA_PREFIX):
-		bits[key[len(s3amazon.S3.METADATA_PREFIX):]] = response_headers[key]
-		del response_headers[key]
-	    elif key.lower() == 'content-length':
-		bits.size = response_headers[key]
-	file = open(filename, 'wb')
-	l = resp.read(self.BufferSize)
-	while len(l) > 0:
-	    file.write(l)
-	    l = resp.read(self.BufferSize)
-	file.close()
-	return resp
+def filter_bits(bits):
+    filter = get_content_filter(bits.content_type)
+    if filter:
+	filter.filter(bits)
 
-    def list_bucket(self, bucket, options={}, headers={}):
-        path = bucket
-        if options:
-            path += '?' + '&'.join(["%s=%s" % (param, urllib.quote_plus(str(options[param]))) for param in options])
+class ContentFilter:
+
+    def __init__(self):
+	self.content_type = None
+
+    def filter(self, bits):
+	print 'filter[%s] - filtering %s' % (self.content_type, bits.filename)
 
-        return BBListBucketResponse(self.make_request('GET', path, headers))
 
 class BBHeadResponse(s3amazon.S3.GetResponse):
     
@@ -177,7 +128,6 @@ class BBListBucketHandler(xml.sax.ContentHandler):
     def characters(self, content):
         self.curr_text += content
 
-
 #
 # Exception classes - Subclassing allows you to check for specific errors
 #
@@ -225,9 +175,8 @@ class Bits(s3amazon.S3.S3Object):
 
     def __init__(self, filename=None):
 	self.state = BITS_LOOSE
-	s3amazon.S3.S3Object.__init__(self, None)
+	self.metadata = {}
 	self.bucket = None
-	# need to get info about file
 	self.filename = filename
 	self.etag = None
 	self.key = None
@@ -240,7 +189,7 @@ class Bits(s3amazon.S3.S3Object):
 	    if self.state == BITS_NEED_READ:
 		if self.bucket:
 		    self.bucket.get_bits(self)
-	    return self.__data__
+	    return self._data
 	elif name in self.metadata:
 	    return self.metadata[name]
 	else:
@@ -248,15 +197,15 @@ class Bits(s3amazon.S3.S3Object):
 
     def __setattr__(self, name, value):
 	if name == 'data':
-	    self.__data__ = value
+	    self._data = value
 	    if value:
 		self.__dict__['size'] = len(value)
-	    if self.state == BITS_NEED_WRITE:
-		if self.bucket:
-		    self.bucket.send_bits(self)
+	    if self.bucket:
+		self.bucket.send_bits(self)
 	elif name == 'filename':
 	    self.__dict__[name] = value
-	    self.sync()
+	    if value:
+		self.sync()
 	else:
 	    self.__dict__[name] = value
 
@@ -269,13 +218,23 @@ class Bits(s3amazon.S3.S3Object):
     def __delitem__(self, key):
 	del self.metadata[key]
 
+    def _compute_md5(self):
+	m = md5.new()
+	p = open(self.filename)
+	s = p.read(8192)
+	while s:
+	    m.update(s)
+	    s = p.read(8192)
+	self.md5 = '"%s"' % m.hexdigest()
+	p.close()
+
     def sync(self):
 	if self.filename:
 	    if os.path.exists(self.filename):
 		self.size = os.stat(self.filename).st_size
-		p = os.popen(MD5Command % self.filename)
-		self.md5 = '"%s"' % string.strip(p.read())
+		self._compute_md5()
 		self.content_type = mimetypes.guess_type(self.filename)[0]
+		filter_bits(self)
 		self.state = BITS_NEED_WRITE
 		if self.bucket:
 		    self.bucket.send_file(self)
@@ -283,6 +242,9 @@ class Bits(s3amazon.S3.S3Object):
 		self.state = BITS_NEED_READ
 		if self.bucket:
 		    self.bucket.get_file(self, self.filename)
+	else:
+	    self.state = BITS_NEED_WRITE
+	    self.bucket.send_bits(self)
 
     def get_url(self, expires_in=60):
 	if self.bucket:
@@ -296,6 +258,14 @@ class Bits(s3amazon.S3.S3Object):
 	else:
 	    raise BitBucketError("Bits aren't associated with a BitBucket yet")
 
+    def set_canned_acl(self, policy):
+	if policy not in _canned_access_policies:
+	    raise BitBucketError('Invalid acl_policy: %s' % policy)
+	self.bucket.connection.set_canned_acl(self, policy)
+
+    def get_acl(self):
+	return self.bucket.connection.get_acl(self)
+
     def to_file(self, filename):
 	if self.bucket:
 	    self.bucket.get_file(self, filename)
@@ -326,108 +296,99 @@ def BitBucketGenerator(bucket):
 
 class BitBucket:
 
-    def __init__(self, name, prefix=None, access_key=None,
-		 secret_key=None, bucket_name_prefix='', page_size=None):
-	global MD5Command
-	self.get_config(access_key, secret_key, bucket_name_prefix, page_size)
+    def __init__(self, name, connection, prefix=None,
+		 page_size=100, debug=None):
 	self.options = {}
+	self.connection = connection
 	if prefix:
 	    self.options['prefix'] = prefix
-	self.name = self.bucket_name_prefix + name
-	self.connection = BBAWSAuthConnection(self.access_key,
-					      self.secret_key)
-	self.query_gen = s3amazon.S3.QueryStringAuthGenerator(self.access_key,
-							      self.secret_key)
-	self.__cache__ = {}
-	self.create_bucket_if_necessary()
+	self.page_size = page_size
+	self.debug = debug
+	self.name = self.connection.bucket_name_prefix + name
+	self._cache = {}
 
-    def get_config(self, access_key, secret_key, bucket_name_prefix, page_size):
-	# check to see if there is a bitbucket.cfg file
-	# if so, we use those values unless values are passed to constructor
-	self.config = ConfigParser.ConfigParser(AWS_DEFAULTS)
-	self.config.read(['./bitbucket.cfg',
-			  os.path.expanduser('~/bitbucket.cfg')])
-	if access_key:
-	    self.access_key = access_key
-	else:
-	    self.access_key = self.config.get(ConfigParser.DEFAULTSECT,
-					      'AccessKeyID')
-	if secret_key:
-	    self.secret_key = secret_key
-	else:
-	    self.secret_key = self.config.get(ConfigParser.DEFAULTSECT,
-					      'SecretAccessKey')
-	if bucket_name_prefix:
-	    self.bucket_name_prefix = bucket_name_prefix
-	else:
-	    self.bucket_name_prefix = self.config.get(ConfigParser.DEFAULTSECT,
-						      'BucketNamePrefix')
-	if page_size:
-	    self.page_size = page_size
-	else:
-	    self.page_size = self.config.getint(ConfigParser.DEFAULTSECT,
-						'PageSize')
-	self.debug = self.config.getboolean(ConfigParser.DEFAULTSECT,
-					    'Debug')
-	MD5Command = self.config.get(ConfigParser.DEFAULTSECT,
-				     'MD5Command')
-
-    def create_bucket_if_necessary(self):
-	self.__resp__ = self.connection.create_bucket(self.name)
-	status = self.__resp__.http_response.status
-	if status != 200 and status != 409:
-	    raise BitBucketResponseError(self.__resp__.http_response.status,
-					 self.__resp__.http_response.reason)
-	    
     def get_head(self, key):
-	self.__resp__ = self.connection.head(self.name, key)
-	if self.__resp__.http_response.status == 200:
+	self._resp = self.connection.head(self.name, key)
+	if self._resp.http_response.status == 200:
 	    bits = Bits()
 	    bits.bucket = self
-	    bits.metadata = self.__resp__.metadata
-	    http_resp = self.__resp__.http_response
+	    bits.metadata = self._resp.metadata
+	    http_resp = self._resp.http_response
 	    bits.etag = http_resp.getheader('etag')
 	    bits.content_type = http_resp.getheader('content-type')
 	    bits.last_modified = http_resp.getheader('last-modified')
 	    bits.state = BITS_NEED_READ
-	    self.__cache__[key] = bits
+	    self._cache[key] = bits
 	    bits.key = key
 	    return bits
-	elif self.__resp__.http_response.status == 404:
+	elif self._resp.http_response.status == 404:
 	    return None
 	else:
-	    raise BitBucketResponseError(self.__resp__.http_response.status,
-					 self.__resp__.http_response.reason)
+	    raise BitBucketResponseError(self._resp.http_response.status,
+					 self._resp.http_response.reason)
 
     def fetch_all_keys(self, last_key=None):
 	if last_key:
 	    self.options['marker'] = last_key
-	self.__resp__ = self.connection.list_bucket(self.name, self.options)
-	if self.__resp__.http_response.status == 200:
-	    for b in self.__resp__.entries:
+	self._resp = self.connection.list_bucket(self.name, self.options)
+	if self._resp.http_response.status == 200:
+	    for b in self._resp.entries:
 		b.state = BITS_NEED_READ
 		b.bucket = self
 		key = self.get_local_key(b.key)
 		b.key = key
-		self.__cache__[b.key] = b
+		self._cache[b.key] = b
 		last_key = key
-	    if self.__resp__.is_truncated == 'true':
+	    if self._resp.is_truncated == 'true':
 		if self.debug:
 		    print '__list__: getting next page - last_key=%s' % last_key
 		self.fetch_all_keys(last_key)
 	else:
-	    raise BitBucketResponseError(self.__resp__.http_response.status,
-					 self.__resp__.http_response.reason)
+	    raise BitBucketResponseError(self._resp.http_response.status,
+					 self._resp.http_response.reason)
 	    
+    def delete_all_keys(self):
+	self.options['max-keys'] = self.page_size
+	finished = False
+	last_key = None
+	while not finished:
+	    if last_key:
+		self.options['marker'] = last_key
+	    resp = self.connection.list_bucket(self.name, self.options)
+	    if resp.http_response.status == 200:
+		for b in resp.entries:
+		    key = b.key
+		    last_key = key
+		    if self.debug:
+			print '[delete_all_keys]: deleting %s' % key
+		    self._resp = self.connection.delete(self.name,
+							self.get_aws_key(key))
+		    if self._resp.http_response.status != 204:
+			raise BitBucketResponseError(self._resp.http_response.status,
+						     self._resp.http_response.reason)
+		if resp.is_truncated != 'true':
+		    finished = True
+	    else:
+		raise BitBucketResponseError(resp.http_response.status,
+					     resp.http_response.reason)
+		
+    def get_acl(self):
+	return self.connection.get_acl(self)
+
+    def set_canned_acl(self, policy):
+	if policy not in _canned_access_policies:
+	    raise BitBucketError('Invalid acl_policy: %s' % policy)
+	self.connection.set_canned_acl(self, policy)
+
     def __repr__(self):
 	return 'BitBucket(%s)' % self.name
 
     def __len__(self):
-	return len(self.__cache__.keys())
+	return len(self._cache.keys())
 
     def __getitem__(self, key):
-	if self.__cache__.has_key(key):
-	    return self.__cache__[key]
+	if self._cache.has_key(key):
+	    return self._cache[key]
 	bits = self.get_head(key)
 	if bits:
 	    return bits
@@ -441,18 +402,18 @@ class BitBucket:
 	    self.send_file(bits)
 	else:
 	    self.send_bits(bits)
-	self.__cache__[bits.key] = bits
+	self._cache[bits.key] = bits
 
     def __delitem__(self, key):
-	del self.__cache__[key]
-	self.__resp__ = self.connection.delete(self.name,
+	del self._cache[key]
+	self._resp = self.connection.delete(self.name,
 					       self.get_aws_key(key))
-	if self.__resp__.http_response.status != 204:
-	    raise BitBucketResponseError(self.__resp__.http_response.status,
-					 self.__resp__.http_response.reason)
+	if self._resp.http_response.status != 204:
+	    raise BitBucketResponseError(self._resp.http_response.status,
+					 self._resp.http_response.reason)
 
     def __contains__(self, item):
-	if item in self.__cache__:
+	if item in self._cache:
 	    return True
 	if self.get_head(key):
 	    return True
@@ -478,11 +439,11 @@ class BitBucket:
 	if bits:
 	    if not isinstance(bits, Bits):
 		raise BitBucketTypeError('Value must be of type Bits')
-	self.query_gen.set_expires_in(expires_in)
+	self.connection.query_gen.set_expires_in(expires_in)
 	if request == 'get':
-	    return self.query_gen.get(self.name, bits.key)
+	    return self.connection.query_gen.get(self.name, bits.key)
 	elif request == 'delete':
-	    return self.query_gen.delete(self.name, bits.key)
+	    return self.connection.query_gen.delete(self.name, bits.key)
 	else:
 	    raise BitBucketError('Invalid request: %s' % request)
 
@@ -498,13 +459,15 @@ class BitBucket:
 	    if self.debug:
 		print '[send_bits] sending %s' % bits.key
 	    headers = {'ETag':etag}
-	    self.__resp__ = self.connection.put(self.name,
+	    if bits.content_type:
+		headers['Content-Type'] = bits.content_type
+	    self._resp = self.connection.put(self.name,
 						self.get_aws_key(bits.key),
 						bits, headers)
-	    if self.__resp__.http_response.status != 200:
-		raise BitBucketResponseError(self.__resp__.http_response.status,
-					 self.__resp__.http_response.reason)
-	    if etag != self.__resp__.http_response.getheader('etag'):
+	    if self._resp.http_response.status != 200:
+		raise BitBucketResponseError(self._resp.http_response.status,
+					 self._resp.http_response.reason)
+	    if etag != self._resp.http_response.getheader('etag'):
 		raise BitBucketDataError("ETags don't match")
 	else:
 	    if self.debug:
@@ -524,11 +487,11 @@ class BitBucket:
 	    headers = {'ETag':bits.md5}
 	    if bits.content_type:
 		headers['Content-Type'] = bits.content_type
-	    self.__resp__ = self.connection.put_file(bits, headers)
-	    if self.__resp__.http_response.status != 200:
-		raise BitBucketResponseError(self.__resp__.http_response.status,
-					 self.__resp__.http_response.reason)
-	    bits.etag = self.__resp__.http_response.getheader('etag')
+	    self._resp = self.connection.put_file(bits, headers)
+	    if self._resp.http_response.status != 200:
+		raise BitBucketResponseError(self._resp.http_response.status,
+					 self._resp.http_response.reason)
+	    bits.etag = self._resp.http_response.getheader('etag')
 	    if bits.etag != bits.md5:
 		raise BitBucketDataError("ETags don't match")
 	else:
@@ -543,48 +506,268 @@ class BitBucket:
 	    raise BitBucketError('Those are not my Bits')
 	if self.debug:
 	    print '[get_bits] getting %s' % bits.key
-	self.__resp__ = self.connection.get(self.name,
+	self._resp = self.connection.get(self.name,
 					    self.get_aws_key(bits.key))
-	if self.__resp__.http_response.status == 200:
-	    bits.__data__ = self.__resp__.object.data
-	    bits.metadata = self.__resp__.object.metadata
+	if self._resp.http_response.status == 200:
+	    bits._data = self._resp.object.data
+	    bits.metadata = self._resp.object.metadata
 	    bits.state = BITS_IN_SYNC
 	else:
-	    raise BitBucketResponseError(self.__resp__.http_response.status,
-					 self.__resp__.http_response.reason)
+	    raise BitBucketResponseError(self._resp.http_response.status,
+					 self._resp.http_response.reason)
 	
     def get_file(self, bits, filename):
 	if not isinstance(bits, Bits):
 	    raise BitBucketTypeError('Value must be of type Bits')
 	if self.debug:
 	    print '[get_file] getting %s' % bits.key
-	self.__resp__ = self.connection.get_file(bits, filename)
-	if self.__resp__.status != 200:
-	    raise BitBucketResponseError(self.__resp__.status,
-					 self.__resp__.reason)
-	bits.etag = self.__resp__.getheader('etag')
+	self._resp = self.connection.get_file(bits, filename)
+	if self._resp.status != 200:
+	    raise BitBucketResponseError(self._resp.status,
+					 self._resp.reason)
+	bits.etag = self._resp.getheader('etag')
 	bits.state = BITS_IN_SYNC
 
     def keys(self):
-	return self.__cache__.keys()
+	return self._cache.keys()
 
     def values(self):
-	return self.__cache__.values()
+	return self._cache.values()
 
     def items(self):
-	return self.__cache__.items()
+	return self._cache.items()
 
     def has_key(self, key):
-	if self.__cache__.has_key(key):
+	if self._cache.has_key(key):
 	    return True
 	if self.get_head(key):
 	    return True
 	else:
 	    return False
 
+#
+# You have a couple of choices for getting your individual keys in.
+#   1. You could pass them to the BitBucket constructor
+#   2. You could add them to this dictionary
+#   3. You could place them in a file called bitbucket.cfg and put that
+#      file in either your home directory or in the directory you are
+#      running the code in.  The file needs to look like this:
+#-------------------------------------------------------
+#      [DEFAULTS]
+#      AccessKeyID: YourAccessKeyHere
+#      SecretAccessKey: YourSecretAccessKeyHere
+#      Debug: 1
+#-------------------------------------------------------
+BB_DEFAULTS = {'AccessKeyID': '',
+	       'SecretAccessKey': '',
+	       'BucketNamePrefix': '',
+	       'PageSize': 100,
+	       'Debug': 1}
 
+#
+# Just wanted to subclass to include an implementation of HEAD
+#
+class BBConnection(s3amazon.S3.AWSAuthConnection):
+
+    BufferSize=8192
+
+    def __init__(self, access_key=None, secret_key=None,
+		 is_secure=True, server=s3amazon.S3.DEFAULT_HOST, port=None,
+		 bucket_name_prefix=None, page_size=None):
+	self._buckets = {}
+	# check to see if there is a bitbucket.cfg file
+	# if so, we use those values unless values are passed to constructor
+	self.config = ConfigParser.ConfigParser(BB_DEFAULTS)
+	self.config.read(['./bitbucket.cfg',
+			  os.path.expanduser('~/bitbucket.cfg')])
+	if access_key:
+	    self.access_key = access_key
+	else:
+	    self.access_key = self.config.get(ConfigParser.DEFAULTSECT,
+					      'AccessKeyID')
+	if secret_key:
+	    self.secret_key = secret_key
+	else:
+	    self.secret_key = self.config.get(ConfigParser.DEFAULTSECT,
+					      'SecretAccessKey')
+	if bucket_name_prefix:
+	    self.bucket_name_prefix = bucket_name_prefix
+	else:
+	    self.bucket_name_prefix = self.config.get(ConfigParser.DEFAULTSECT,
+						      'BucketNamePrefix')
+	if page_size:
+	    self.page_size = page_size
+	else:
+	    self.page_size = self.config.getint(ConfigParser.DEFAULTSECT,
+						'PageSize')
+	self.debug = self.config.getint(ConfigParser.DEFAULTSECT,
+					'Debug')
+	s3amazon.S3.AWSAuthConnection.__init__(self, self.access_key,
+					       self.secret_key, is_secure,
+					       server, port)
+	# if debug>1 then turn on httplib debug as well
+	if self.debug > 1:
+	    self.connection.debuglevel = self.debug
+	self.query_gen = s3amazon.S3.QueryStringAuthGenerator(self.access_key,
+							      self.secret_key)
+	self.get_all_buckets()
     
+    def __len__(self):
+	return len(self._buckets.keys())
 
-        
-	    
+    def __getitem__(self, key):
+	if self._buckets.has_key(key):
+	    return self._buckets[key]
+	else:
+	    raise KeyError(key)
 
+    def __delitem__(self, key):
+	del self._buckets[key]
+	self._resp = self.connection.delete_bucket(self.name)
+	if self._resp.http_response.status != 204:
+	    raise BitBucketResponseError(self._resp.http_response.status,
+					 self._resp.http_response.reason)
+
+    def __contains__(self, item):
+	if item in self._buckets:
+	    return True
+	else:
+	    return False
+
+    def keys(self):
+	return self._buckets.keys()
+
+    def values(self):
+	return self._buckets.values()
+
+    def items(self):
+	return self._buckets.items()
+
+    def has_key(self, key):
+	if self._buckets.has_key(key):
+	    return True
+	else:
+	    return False
+	
+    def head(self, bucket, key, headers={}):
+        return BBHeadResponse(
+	    self.make_request('HEAD', '%s/%s' % (bucket, urllib.quote_plus(key)), headers))
+
+    #
+    # This method mainly stolen from Emanuele Ruffaldi's S3 Tool
+    #
+    def put_file(self, bits, headers={}):
+	if not isinstance(bits, Bits):
+	    raise BitBucketTypeError('Must pass Bits to put_file')
+	final_headers = s3amazon.S3.merge_meta(headers, bits.metadata);
+	path = '%s/%s' % (bits.bucket.name, urllib.quote_plus(bits.key))
+	self.add_aws_auth_header(final_headers, 'PUT', path)
+	self.connection.putrequest('PUT','/'+path)
+	final_headers["Content-Length"] = bits.size
+	for key in final_headers:
+	    self.connection.putheader(key,final_headers[key])
+	self.connection.endheaders()
+	file = open(bits.filename, 'rb')
+	l = file.read(self.BufferSize)
+	while len(l) > 0:
+	    self.connection.send(l)
+	    l = file.read(self.BufferSize)
+	file.close()
+	return s3amazon.S3.Response(self.connection.getresponse())
+
+    def get_file(self, bits, filename, headers={}):
+	if not isinstance(bits, Bits):
+	    raise BitBucketTypeError('Must pass Bits to get_file')
+	final_headers = s3amazon.S3.merge_meta(headers, bits.metadata);
+	path = '%s/%s' % (bits.bucket.name, urllib.quote_plus(bits.key))
+	self.add_aws_auth_header(final_headers, 'GET', path)
+	self.connection.putrequest('GET','/'+path)
+	for key in final_headers:
+	    self.connection.putheader(key,final_headers[key])
+	self.connection.endheaders()
+	resp = self.connection.getresponse()
+	response_headers = resp.msg
+	for key in response_headers.keys():
+	    if key.lower().startswith(s3amazon.S3.METADATA_PREFIX):
+		bits[key[len(s3amazon.S3.METADATA_PREFIX):]] = response_headers[key]
+		del response_headers[key]
+	    elif key.lower() == 'content-length':
+		bits.size = response_headers[key]
+	file = open(filename, 'wb')
+	l = resp.read(self.BufferSize)
+	while len(l) > 0:
+	    file.write(l)
+	    l = resp.read(self.BufferSize)
+	file.close()
+	return resp
+
+    def list_bucket(self, bucket, options={}, headers={}):
+        path = bucket
+        if options:
+            path += '?' + '&'.join(["%s=%s" % (param, urllib.quote_plus(str(options[param]))) for param in options])
+
+        return BBListBucketResponse(self.make_request('GET', path, headers))
+
+    def get_all_buckets(self):
+	r = self.list_all_my_buckets()
+	for entry in r.entries:
+	    b = BitBucket(entry.name, self,
+			  page_size=self.page_size, debug=self.debug)
+	    self._buckets[entry.name] = b
+
+    def get_bucket(self, bucket_name, prefix=None):
+	if bucket_name in self._buckets.keys():
+	    return self._buckets[bucket_name]
+	self._resp = s3amazon.S3.AWSAuthConnection.create_bucket(self,
+								 bucket_name)
+	status = self._resp.http_response.status
+	if status == 409:
+	    raise BitBucketCreateError(self._resp.http_response.status,
+				       self._resp.http_response.reason)
+	if status == 200:
+	    b = BitBucket(bucket_name, self,
+			  page_size=self.page_size, debug=self.debug)
+	    self._buckets[bucket_name] = b
+	    return b
+	else:
+	    raise BitBucketResponseError(self._resp.http_response.status,
+					 self._resp.http_response.reason)
+
+    def delete_bucket(self, bucket_name):
+	self._resp = s3amazon.S3.AWSAuthConnection.delete_bucket(self,
+								 bucket_name)
+	status = self._resp.http_response.status
+	if status == 204:
+	    if bucket_name in self._buckets.keys():
+		del self._buckets[bucket_name]
+	else:
+	    raise BitBucketResponseError(self._resp.http_response.status,
+					 self._resp.http_response.reason)
+
+    def get_acl(self, b):
+	if isinstance(b, Bits):
+	    b_name = b.bucket.name
+	    key = b.key
+	elif isinstance(b, BitBucket):
+	    b_name = b.name
+	    key = ''
+	else:
+	    raise BitBucketError('Must pass Bits or BitBucket object')
+	self._resp = s3amazon.S3.AWSAuthConnection.get_acl(self, b_name, key)
+	return self._resp.body
+
+    def set_canned_acl(self, b, policy):
+	if isinstance(b, Bits):
+	    b_name = b.bucket.name
+	    key = b.key
+	elif isinstance(b, BitBucket):
+	    b_name = b.name
+	    key = ''
+	else:
+	    raise BitBucketError('Must pass Bits or BitBucket object')
+	headers = {'x-amz-acl': policy}
+	self._resp = s3amazon.S3.AWSAuthConnection.put_acl(self, b_name,
+							   key, '', headers)
+		
+def connect(access_key=None, secret_key=None):
+    return BBConnection(access_key, secret_key)
